###########################################################################
#!make
#
#!Makefile Name: MOD_PR04_05.mk
#
#!Description: Makefile for compiling the V2 Aerosol/Water Vapor product
#              generation software (MOD_PR04_05).
#
#!Variables:
#
#       Internal
#        Macros          Definition
#       ~~~~~~~~         ~~~~~~~~~~
#       ADD_CFLAGS       Process specific C compiler options
#       ADD_F77FLAGS     Process specific FORTRAN 77 compiler options
#       ADD_F90FLAGS     Process specific FORTRAN 90 compiler options
#       HDF5OPT_LIB       Path name of HDF5 dynamically shared library
#       INC              Accumlated include file search path
#       INC_FILE_SCIENCE Include files developed by science team
#       INC_FILE_SHARED  Atmosphere shared include files
#       LIB              Accumulated library search path
#       OBJ              Accumulated set of object files
#       PATH_SRC_SDST    Directory path to SDST L2 atmosphere shared source
#       PATH_SRC_UW      Directory path to UW-Madison shared source
#       TARGET           Executable file name
#
#!Env Variables
#
#         System
#       Environment
#        Variables    Definition
#       ~~~~~~~~~~~   ~~~~~~~~~~
#       API_INC       Directory path to M-API include files
#       API_LIB       Directory path to M-API library
#       BRAND         Compiler brand. Current valids are "sgi64/sgi32/linux".
#                     "linux" denotes PGI compilers for FORTRAN code and GNU
#                     compilers for C.  
#       CC            C compiler definition
#       CFLAGS        MODIS common C compiler flags
#       F77           FORTRAN 77 compiler definition
#       F77FLAGS      MODIS common FORTRAN 77 compiler flags
#       F90           FORTRAN 90 compiler definition 
#       F90FLAGS      Common MODIS FORTRAN 90 compiler options
#       HDFEOS_INC    Directory path to HDFEOS include files
#       HDFEOS_LIB    Directory path to HDFEOS library
#       HDFINC        Directory path to HDF include files
#       HDFLIB        Directory path to HDF library
#       HDF5LIB       Directory path to HDF5 libraries
#       PGSINC        Directory path to SDPTK include files
#       PGSLIB        Directory path to SDPTK library
#
#        User
#     Environment
#      Variables      Definition
#     ~~~~~~~~~~~     ~~~~~~~~~~
#       MODIS_STORE   Home directory path to MODIS source code
#
#
#!Team-Unique Header:
#       This software was developed by MODIS Science Data Support Team
#       for the National Aeronautics and Space Administration,
#       Goddard Space Flight Center, under contract NAS5-32373.
#
#!Revision History:
# 12/17/98 rhucek
# change to comply with MODIS makefile standards.
#
#!Note:
#       1) must follow MAPI, SDP TK, HDF, and other header/libs order
#       2) HDFLIB must be explicit and in order of -lmfhdf, -ldf, -ljpeg, -lz.
#       3) system environments, such as API_INC, PGSINC, HDFINC, PGSLIB,
#          API_LIB etc. should be defined before executing this makefile.
#
#!END
###########################################################################

# Define internal macros
PATH_SRC_SDST = $(MODIS_STORE)/shared_src/atmos_src/src_L2
PATH_SRC_UW   = $(MODIS_STORE)/shared_src/atmos_src/src_UW

# Objects for all
OBJ.0 = MOD_PR04_PR05_V2.o           Read_CldMsk_V2.o \
        Utility_V2.o                 Gen_Modis_FileName.o \
        Get_date_time.o              Read_L1B_V2.1.o \
        MODIS_IO_GEN_OPEN.o          MODIS_IO_GEN_CLOSE.o \
        Set_ArchiveMetadata_V2.1.o   Set_CoreMetadata_QC.o \
        Set_CoreMetadata_V2.1.o      chk_input_L2.o \
        strlen.o                     addtext.o \
        STRING_LOC.o                 MODIS_SMF_SETDYNAMICMSG_V2.1.o \
        Met_Common_Atmos.o \
        CONCATENATE.o

# Objects for MOD04
OBJ.4 = MOD_PR04_V2.o \
        create_mod04_V2.o \
        GetModisData_MOD04_V2.o \
        mod04_file_open_V2.o \
        mod04_file_close_V2.o \
        mod04_out_V2.o \
        Process_ocean_V2.o \
        Process_land_V2.o \
        GetAncData_PGE04.o \
        read_modisdep_V2.o \
        Fillvalue_MOD04_V2.o \
        MetaData_MOD04_V2.o \
        modis_grib_read.o \
        modis_grib_driver_1.o \
        message_SDST.o \
        strcompress.o \
        modis_io_gen_temp_openf.o \
        svpwat.o \
        Delete_CommonTempFiles_Atmos_V2.1.o \
        Delete_PCF_TempFile.o \
        openr_temp_SDST.o

# Objects for MOD05
OBJ.5 = MOD_PR05_V2.o \
        MOD05_IO_V2.o \
        GetModisData_MOD05_V2.o \
        mod05_fileoc_V2.o \
        create_mod05_V2.o \
        MetaData_MOD05_V2.o \
        MOD05_Process_Night.o \
        Copy_Swath_SDS.o


# Objects for aerosol correction on total precipitable water
OBJ.5C = MOD_PR05_CORR_V2.o \
         GetModisData_MOD05_CORR_V2.o \
         MOD05_CORR_fileoc_V2.o

# Objects for MOD06CD
OBJ.6 = MOD_PR06CD_V2.o \
        Process_mod06cd_V2.o \
        cirrus_fileoc.o \
        GetModisDat_cirrus_V2.o


OBJ = $(OBJ.0) $(OBJ.4) $(OBJ.5) $(OBJ.5C) $(OBJ.6)


# Include file search path
INC = -I$(API_INC) \
      -I$(HDFINC) \
      -I$(PGSINC) \
      -I$(HDFEOS_INC) \
      -I$(PATH_SRC_SDST) \
      -I$(PATH_SRC_UW)

# Include files developed by ST
INC_FILES_SCIENCE = COMMONS.inc \
		    MOD05_CORR.inc \
                    mod04.inc mapi_hdfeos.inc \
                    mod05.inc mod05_hdfeos.inc \
                    COMMONS_cirrus.inc \
                    cirrus.inc

# Include files from MODIS shared source
INC_FILES_SHARED  = $(PATH_SRC_SDST)/L1B_Reader_V2.1.inc \
		    $(PATH_SRC_SDST)/ckstatus.inc \
		    $(PATH_SRC_UW)/modis_grib_read.h \
		    $(PATH_SRC_SDST)/Atmos_AncData.inc


# Libraries 
# Define library search path; it includes libhdf5.so for SDPTK 5.2.7.3 and above.
# Note the use of make substitution operator: $(variable:pattern=replacement). If
# environment variable $(HDF5LIB) is undefined, HDF5OPT_LIB is set NULL. 

HDF5OPT_LIB = /$(HDF5LIB)/libhdf5.so
LIB         = -L$(API_LIB) -lmapi \
              -L$(PGSLIB) -lPGSTK -lm \
              -L$(HDFEOS_LIB) -lhdfeos \
              $(HDF5OPT_LIB://libhdf5.so=) \
              -L$(HDFLIB) -lmfhdf -ldf -ljpeg -lz


###########################################################################
#                           debug flags and environment
#
# -trapuv:        identifies usage of undefined variables as "INVALID"
# -C:             terminates program at point of subscript out-of-range.
# -check_bounds:  same as -C
# -g:             used to run in UNIX debugger, dbx
# -lfpe:          collects floating point statistics as defined in
#                 environment variable "TRAP_FPE".  E.g., set TRAP_FPE
#                 setenv TRAP_FPE "ALL=COUNT,TRACE(1)"
#
# test 1:  run from UNIX command line: perform subscript range checking,
#          count occurrences of floating point exception (fpe) types including
#          INVALID, DIVZERO, UNDERFL, OVERFL.  Trace first occurrence of
#          each fpe type and print to standard output.
#ADD_CFLAGS   = -O0 -OPT:Olimit=0 -trapuv -C               -c -r10000 -lfpe
#ADD_F77FLAGS = -O0 -OPT:Olimit=0 -trapuv -C -check_bounds -c -r10000 -col120 -bytereclen -lfpe
#ADD_F90FLAGS = -O0 -OPT:Olimit=0 -trapuv -C -check_bounds -c -r10000 -ansi -fullwarn -lfpe
#setenv TRAP_FPE "ALL=COUNT,TRACE(1)"
#
# test 2: like test 1, but with full access to UNIX debugger features available.
#ADD_CFLAGS   = -g -O0 -OPT:Olimit=0 -trapuv -C               -c -r10000 -lfpe
#ADD_F77FLAGS = -g -O0 -OPT:Olimit=0 -trapuv -C -check_bounds -c -r10000 -col120 -bytereclen -lfpe
#ADD_F90FLAGS = -g -O0 -OPT:Olimit=0 -trapuv -C -check_bounds -c -r10000 -ansi -fullwarn -lfpe
#setenv TRAP_FPE "ALL=COUNT,TRACE(1)"
###########################################################################

# Compiler options are set based on IRIX/Linux environment.  On Linux, 
# FORTRAN codes are compiled using PGI compilers; C codes use the GNU 
# compilers 
target :
	case "$(BRAND)" in \
          sgi32 | sgi64 ) \
            $(MAKE) $(TARGET) -f MOD_PR04_05.mk \
                    ADD_F77FLAGS=" -c -O2 -OPT:Olimit=0 -col120" \
                    ADD_F90FLAGS=" -c -O2 -OPT:Olimit=0 -col120" \
                    ADD_CFLAGS="   -c -O2 -OPT:Olimit=0" \
            ;; \
          linux ) \
            $(MAKE) $(TARGET) -f MOD_PR04_05.mk \
                    ADD_F77FLAGS=" -c -O2 -Mstandard" \
                    ADD_F90FLAGS=" -c -O2 -Mstandard" \
                    ADD_CFLAGS="   -c -O2 -funsigned-char" \
            ;; \
          * ) \
            echo "  ERROR: Environment variable BRAND (set to \"$(BRAND)\") not recognized." >&2 ; \
            exit 1 ;; \
        esac


TARGET = MOD_PR04_05.exe

# Make executable
$(TARGET): $(OBJ)
	$(F90) $(OBJ) $(LIB) -o $@
 

#----------------------------------------------------------------------------------
# Build object modules from UW-Madison shared source library files
#----------------------------------------------------------------------------------

modis_grib_read.o: $(PATH_SRC_UW)/modis_grib_read.c $(INC_FILES_SHARED)
	$(CC) $(CFLAGS) $(ADD_CFLAGS) $(INC) $(PATH_SRC_UW)/modis_grib_read.c

modis_io_gen_temp_openf.o: $(PATH_SRC_UW)/modis_io_gen_temp_openf.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/modis_io_gen_temp_openf.f

strcompress.o: $(PATH_SRC_UW)/strcompress.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/strcompress.f

svpwat.o: $(PATH_SRC_UW)/svpwat.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/svpwat.f


#-----------------------------------------------------------------------------
# Build object modules from SDST shared source library files
#-----------------------------------------------------------------------------
 
addtext.o: $(PATH_SRC_SDST)/addtext.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/addtext.f

chk_input_L2.o: $(PATH_SRC_SDST)/chk_input_L2.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/chk_input_L2.f

CONCATENATE.o: $(PATH_SRC_SDST)/CONCATENATE.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/CONCATENATE.f

Copy_Swath_SDS.o: $(PATH_SRC_SDST)/Copy_Swath_SDS.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/Copy_Swath_SDS.f

Delete_CommonTempFiles_Atmos_V2.1.o: $(PATH_SRC_SDST)/Delete_CommonTempFiles_Atmos_V2.1.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/Delete_CommonTempFiles_Atmos_V2.1.f

Delete_PCF_TempFile.o: $(PATH_SRC_SDST)/Delete_PCF_TempFile.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/Delete_PCF_TempFile.f

Gen_Modis_FileName.o: $(PATH_SRC_SDST)/Gen_Modis_FileName.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/Gen_Modis_FileName.f

Get_date_time.o: $(PATH_SRC_SDST)/Get_date_time.f
	$(F90) $(F90FLAGS) $(ADD_F90FLAGS) $(PATH_SRC_SDST)/Get_date_time.f

message_SDST.o: $(PATH_SRC_SDST)/message_SDST.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/message_SDST.f

Met_Common_Atmos.o: $(PATH_SRC_SDST)/Met_Common_Atmos.f $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/Met_Common_Atmos.f

modis_grib_driver_1.o: $(PATH_SRC_SDST)/modis_grib_driver_1.c $(INC_FILES_SHARED)
	$(CC) $(CFLAGS) $(ADD_CFLAGS) $(INC) $(PATH_SRC_SDST)/modis_grib_driver_1.c

MODIS_IO_GEN_OPEN.o: $(PATH_SRC_SDST)/MODIS_IO_GEN_OPEN.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/MODIS_IO_GEN_OPEN.f

MODIS_IO_GEN_CLOSE.o: $(PATH_SRC_SDST)/MODIS_IO_GEN_CLOSE.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/MODIS_IO_GEN_CLOSE.f

MODIS_SMF_SETDYNAMICMSG_V2.1.o: $(PATH_SRC_SDST)/MODIS_SMF_SETDYNAMICMSG_V2.1.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/MODIS_SMF_SETDYNAMICMSG_V2.1.f

Read_CldMsk_V2.o: $(PATH_SRC_SDST)/Read_CldMsk_V2.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/Read_CldMsk_V2.f

Read_L1B_V2.1.o: $(PATH_SRC_SDST)/Read_L1B_V2.1.f $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/Read_L1B_V2.1.f

Set_ArchiveMetadata_V2.1.o: $(PATH_SRC_SDST)/Set_ArchiveMetadata_V2.1.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/Set_ArchiveMetadata_V2.1.f

Set_CoreMetadata_QC.o: $(PATH_SRC_SDST)/Set_CoreMetadata_QC.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/Set_CoreMetadata_QC.f

Set_CoreMetadata_V2.1.o: $(PATH_SRC_SDST)/Set_CoreMetadata_V2.1.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/Set_CoreMetadata_V2.1.f

strlen.o: $(PATH_SRC_SDST)/strlen.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/strlen.f

STRING_LOC.o: $(PATH_SRC_SDST)/STRING_LOC.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/STRING_LOC.f

openr_temp_SDST.o: $(PATH_SRC_SDST)/openr_temp_SDST.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/openr_temp_SDST.f


#----------------------------------------------------
# Build object modules from PGE specific source code 
#----------------------------------------------------

Fillvalue_MOD04_V2.o: Fillvalue_MOD04_V2.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) Fillvalue_MOD04_V2.f

GetModisDat_cirrus_V2.o: GetModisDat_cirrus_V2.f $(INC_FILES_SCIENCE) $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) GetModisDat_cirrus_V2.f

GetModisData_MOD04_V2.o: GetModisData_MOD04_V2.f $(INC_FILES_SCIENCE) $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) GetModisData_MOD04_V2.f

GetModisData_MOD05_CORR_V2.o: GetModisData_MOD05_CORR_V2.f $(INC_FILES_SCIENCE) $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) GetModisData_MOD05_CORR_V2.f

GetModisData_MOD05_V2.o: GetModisData_MOD05_V2.f $(INC_FILES_SCIENCE) $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) GetModisData_MOD05_V2.f

MOD05_CORR_fileoc_V2.o: MOD05_CORR_fileoc_V2.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) MOD05_CORR_fileoc_V2.f

MOD05_IO_V2.o: MOD05_IO_V2.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) MOD05_IO_V2.f

MOD05_Process_Night.o: MOD05_Process_Night.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) MOD05_Process_Night.f

MOD_PR04_PR05_V2.o: MOD_PR04_PR05_V2.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) MOD_PR04_PR05_V2.f

MOD_PR04_V2.o: MOD_PR04_V2.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) MOD_PR04_V2.f

MOD_PR05_CORR_V2.o: MOD_PR05_CORR_V2.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) MOD_PR05_CORR_V2.f

MOD_PR05_V2.o: MOD_PR05_V2.f $(INC_FILES_SCIENCE) $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) MOD_PR05_V2.f

MOD_PR06CD_V2.o: MOD_PR06CD_V2.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) MOD_PR06CD_V2.f

MetaData_MOD04_V2.o: MetaData_MOD04_V2.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) MetaData_MOD04_V2.f

MetaData_MOD05_V2.o: MetaData_MOD05_V2.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) MetaData_MOD05_V2.f

Process_land_V2.o: Process_land_V2.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) Process_land_V2.f

Process_mod06cd_V2.o: Process_mod06cd_V2.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) Process_mod06cd_V2.f

Process_ocean_V2.o: Process_ocean_V2.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) Process_ocean_V2.f

Utility_V2.o: Utility_V2.f $(INC_FILES_SCIENCE) $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) Utility_V2.f

cirrus_fileoc.o: cirrus_fileoc.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) cirrus_fileoc.f

create_mod04_V2.o: create_mod04_V2.f $(INC_FILES_SCIENCE) $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) create_mod04_V2.f

create_mod05_V2.o: create_mod05_V2.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) create_mod05_V2.f

mod04_file_close_V2.o: mod04_file_close_V2.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) mod04_file_close_V2.f

mod04_file_open_V2.o: mod04_file_open_V2.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) mod04_file_open_V2.f

mod04_out_V2.o: mod04_out_V2.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) mod04_out_V2.f

mod05_fileoc_V2.o: mod05_fileoc_V2.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) mod05_fileoc_V2.f

GetAncData_PGE04.o: GetAncData_PGE04.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) GetAncData_PGE04.f

read_modisdep_V2.o: read_modisdep_V2.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) read_modisdep_V2.f

#********************** END OF MAKEFILE ***************************
